@@ -59,18 +59,6 @@ G_DEFINE_TYPE (CheeseCamera, cheese_camera, G_TYPE_OBJECT)
 
 #define CHEESE_CAMERA_ERROR cheese_camera_error_quark ()
 
-typedef enum {
-  GST_CAMERABIN_FLAG_SOURCE_RESIZE               = (1 << 0),
-  GST_CAMERABIN_FLAG_SOURCE_COLOR_CONVERSION     = (1 << 1),
-  GST_CAMERABIN_FLAG_VIEWFINDER_COLOR_CONVERSION = (1 << 2),
-  GST_CAMERABIN_FLAG_VIEWFINDER_SCALE            = (1 << 3),
-  GST_CAMERABIN_FLAG_AUDIO_CONVERSION            = (1 << 4),
-  GST_CAMERABIN_FLAG_DISABLE_AUDIO               = (1 << 5),
-  GST_CAMERABIN_FLAG_IMAGE_COLOR_CONVERSION      = (1 << 6),
-  GST_CAMERABIN_FLAG_VIDEO_COLOR_CONVERSION      = (1 << 7)
-} GstCameraBinFlags;
-
-
 struct _CheeseCameraPrivate
 {
   GstBus *bus;
@@ -88,13 +76,11 @@ struct _CheeseCameraPrivate
 
   ClutterTexture *video_texture;
 
-  GstElement *effect_filter;
-  GstElement *video_balance, *csp_post_balance;
+  GstElement *effect_filter, *effects_capsfilter;
+  GstElement *video_balance;
   GstElement *camera_tee, *effects_tee;
   GstElement *main_valve, *effects_valve;
 
-  GstCaps *preview_caps;
-
   gboolean is_recording;
   gboolean pipeline_is_playing;
   gchar *photo_filename;
@@ -400,6 +386,9 @@ cheese_camera_set_camera_source (CheeseCamera *camera)
   guint               i;
   CheeseCameraDevice *selected_camera;
 
+  if (priv->video_source)
+    gst_object_unref (priv->video_source);
+
   /* If we have a matching video device use that one, otherwise use the first */
   priv->selected_device = 0;
   selected_camera       = g_ptr_array_index (priv->camera_devices, 0);
@@ -417,23 +406,18 @@ cheese_camera_set_camera_source (CheeseCamera *camera)
   }
 
   camera_input = g_strdup_printf (
-    "%s name=video_source device=%s",
+    "%s name=video_source device=%s ! capsfilter name=video_source_filter",
     cheese_camera_device_get_src (selected_camera),
     cheese_camera_device_get_device_node (selected_camera));
 
   priv->video_source = gst_parse_bin_from_description (camera_input, TRUE, &err);
   g_free (camera_input);
 
-  if (priv->video_source == NULL || priv->camera_source == NULL)
+  if (priv->video_source == NULL)
   {
-    if (err != NULL)
-    {
-      g_error_free (err);
-      err = NULL;
-    }
+    g_clear_error(&err);
     return FALSE;
   }
-  g_object_set (priv->camera_source, "video-source", priv->video_source, NULL);
 
   return TRUE;
 }
@@ -532,27 +516,39 @@ cheese_camera_create_effects_preview_bin (CheeseCamera *camera, GError **error)
   CheeseCameraPrivate *priv = camera->priv;
 
   gboolean ok = TRUE;
+  GstElement *scale;
   GstPad  *pad;
 
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   priv->effects_preview_bin = gst_bin_new ("effects_preview_bin");
 
-  if ((priv->effects_tee = gst_element_factory_make ("tee", "effects_tee")) == NULL)
-  {
-    cheese_camera_set_error_element_not_found (error, "tee");
-    return FALSE;
-  }
   if ((priv->effects_valve = gst_element_factory_make ("valve", "effects_valve")) == NULL)
   {
     cheese_camera_set_error_element_not_found (error, "effects_valve");
     return FALSE;
   }
+  if ((scale = gst_element_factory_make ("videoscale", "effects_scale")) == NULL)
+  {
+    cheese_camera_set_error_element_not_found (error, "videoscale");
+    return FALSE;
+  }
+  if ((priv->effects_capsfilter = gst_element_factory_make ("capsfilter", "effects_capsfilter")) == NULL)
+  {
+    cheese_camera_set_error_element_not_found (error, "capsfilter");
+    return FALSE;
+  }
+  if ((priv->effects_tee = gst_element_factory_make ("tee", "effects_tee")) == NULL)
+  {
+    cheese_camera_set_error_element_not_found (error, "tee");
+    return FALSE;
+  }
 
-  gst_bin_add_many (GST_BIN (priv->effects_preview_bin),
-		    priv->effects_valve, priv->effects_tee, NULL);
+  gst_bin_add_many (GST_BIN (priv->effects_preview_bin), priv->effects_valve,
+                    scale, priv->effects_capsfilter, priv->effects_tee, NULL);
 
-  ok &= gst_element_link_many (priv->effects_valve, priv->effects_tee, NULL);
+  ok &= gst_element_link_many (priv->effects_valve, scale,
+                           priv->effects_capsfilter, priv->effects_tee, NULL);
 
   /* add ghostpads */
 
@@ -608,29 +604,22 @@ cheese_camera_create_video_filter_bin (CheeseCamera *camera, GError **error)
     cheese_camera_set_error_element_not_found (error, "videobalance");
     return FALSE;
   }
-  if ((priv->csp_post_balance = gst_element_factory_make ("videoconvert", "csp_post_balance")) == NULL)
-  {
-    cheese_camera_set_error_element_not_found (error, "videoconvert");
-    return FALSE;
-  }
 
   if (error != NULL && *error != NULL)
     return FALSE;
 
   gst_bin_add_many (GST_BIN (priv->video_filter_bin), priv->camera_tee,
                     priv->main_valve, priv->effect_filter,
-                    priv->video_balance, priv->csp_post_balance,
-                    priv->effects_preview_bin, NULL);
+                    priv->video_balance, priv->effects_preview_bin, NULL);
 
   ok &= gst_element_link_many (priv->camera_tee, priv->main_valve,
-                               priv->effect_filter, priv->video_balance,
-                               priv->csp_post_balance, NULL);
+                               priv->effect_filter, priv->video_balance, NULL);
   gst_pad_link (gst_element_get_request_pad (priv->camera_tee, "src_%u"),
                 gst_element_get_static_pad (priv->effects_preview_bin, "sink"));
 
   /* add ghostpads */
 
-  pad = gst_element_get_static_pad (priv->csp_post_balance, "src");
+  pad = gst_element_get_static_pad (priv->video_balance, "src");
   gst_element_add_pad (priv->video_filter_bin, gst_ghost_pad_new ("src", pad));
   gst_object_unref (GST_OBJECT (pad));
 
@@ -744,6 +733,8 @@ cheese_camera_set_new_caps (CheeseCamera *camera)
   CheeseCameraPrivate *priv;
   CheeseCameraDevice *device;
   GstCaps *caps;
+  gchar *caps_desc;
+  int width, height;
 
   g_return_if_fail (CHEESE_IS_CAMERA (camera));
 
@@ -762,8 +753,25 @@ cheese_camera_set_new_caps (CheeseCamera *camera)
 
   if (!gst_caps_is_empty (caps))
   {
-    GST_INFO_OBJECT (camera, "SETTING caps%" GST_PTR_FORMAT, caps);
-    g_object_set (priv->camerabin, "viewfinder-caps", caps, NULL);
+    GST_INFO_OBJECT (camera, "SETTING caps %" GST_PTR_FORMAT, caps);
+    g_object_set (gst_bin_get_by_name (GST_BIN (priv->video_source),
+                  "video_source_filter"), "caps", caps, NULL);
+    g_object_set (priv->camerabin, "viewfinder-caps", caps,
+                  "image-capture-caps", caps, "video-capture-caps", caps,
+                  NULL);
+    gst_caps_unref (caps);
+
+    width = priv->current_format->width;
+    width = width > 640 ? 640 : width;
+    height = width * priv->current_format->height
+             / priv->current_format->width;
+    /* GStreamer will crash if this is not a multiple of 2! */
+    height = (height + 1) & ~1;
+    caps_desc = g_strdup_printf ("video/x-raw, width=%d, height=%d", width,
+                                 height);
+    caps = gst_caps_from_string (caps_desc);
+    g_free (caps_desc);
+    g_object_set (priv->effects_capsfilter, "caps", caps, NULL);
   }
   gst_caps_unref (caps);
 }
@@ -773,6 +781,7 @@ cheese_camera_play (CheeseCamera *camera)
 {
   CheeseCameraPrivate *priv   = CHEESE_CAMERA_GET_PRIVATE (camera);
   cheese_camera_set_new_caps (camera);
+  g_object_set (priv->camera_source, "video-source", priv->video_source, NULL);
   gst_element_set_state (priv->camerabin, GST_STATE_PLAYING);
   priv->pipeline_is_playing = TRUE;
 }
@@ -871,7 +880,7 @@ cheese_camera_element_from_effect (CheeseCamera *camera, CheeseEffect *effect)
   g_free (effects_pipeline_desc);
   if (!effect_filter || (err != NULL))
   {
-    g_error_free (err);
+    g_clear_error (&err);
     g_warning ("Error with effect filter %s. Ignored", name);
     g_free (name);
     return NULL;
@@ -910,7 +919,10 @@ cheese_camera_set_effect (CheeseCamera *camera, CheeseEffect *effect)
 
   g_return_if_fail (CHEESE_IS_CAMERA (camera));
 
-  effect_filter = cheese_camera_element_from_effect (camera, effect);
+  if (strcmp (cheese_effect_get_pipeline_desc (effect), "identity") == 0)
+    effect_filter = gst_element_factory_make ("identity", "effect");
+  else
+    effect_filter = cheese_camera_element_from_effect (camera, effect);
   if (effect_filter != NULL)
     cheese_camera_change_effect_filter (camera, effect_filter);
 }
@@ -934,7 +946,8 @@ cheese_camera_toggle_effects_pipeline (CheeseCamera *camera, gboolean active)
   if (active)
   {
     g_object_set (G_OBJECT (priv->effects_valve), "drop", FALSE, NULL);
-    g_object_set (G_OBJECT (priv->main_valve), "drop", TRUE, NULL);
+    if (!priv->is_recording)
+      g_object_set (G_OBJECT (priv->main_valve), "drop", TRUE, NULL);
   }
   else
   {
@@ -1320,6 +1333,7 @@ cheese_camera_class_init (CheeseCameraClass *klass)
   /**
    * CheeseCamera::photo-taken:
    * @camera: a #CheeseCamera
+   * @pixbuf: a #GdkPixbuf of the photo which was taken
    *
    * Emitted when a photo was taken.
    */
@@ -1346,6 +1360,7 @@ cheese_camera_class_init (CheeseCameraClass *klass)
   /**
    * CheeseCamera::state-flags-changed:
    * @camera: a #CheeseCamera
+   * @state: the #GstState which @camera changed to
    *
    * Emitted when the state of the @camera #GstElement changed.
    */
