@@ -217,14 +217,14 @@ cheese_widget_init (CheeseWidget *widget)
   /* Webcam page */
   priv->screen = gtk_clutter_embed_new ();
   stage = gtk_clutter_embed_get_stage (GTK_CLUTTER_EMBED (priv->screen));
-  clutter_stage_set_color (CLUTTER_STAGE (stage), &black);
+  clutter_actor_set_background_color (stage, &black);
   frame = totem_aspect_frame_new ();
 
   priv->texture = clutter_texture_new ();
   clutter_actor_set_size (priv->texture, 400, 300);
   totem_aspect_frame_set_child (TOTEM_ASPECT_FRAME (frame), priv->texture);
 
-  clutter_container_add_actor (CLUTTER_CONTAINER (stage), frame);
+  clutter_actor_add_child (stage, frame);
   constraint = clutter_bind_constraint_new (stage, CLUTTER_BIND_SIZE, 0.0);
   clutter_actor_add_constraint_with_name (frame, "size", constraint);
 
